'''
    This script takes the csv file cleaned previously as input and
    outputs a clean csv file
'''

import csv, string, re
from nltk.corpus import stopwords
from nltk.stem.snowball import SnowballStemmer

# csv files path
IN = 'almost clean csv file'
OUT = 'clean csv file'

# some costants to speed up the process
PATTERN = re.compile('[^\x00-\x7f]')
STOP_WORDS = stopwords.words('english')
SNOW = SnowballStemmer('english')
PUNCT = string.punctuation

def cleanPost(post): 
    post = post.translate(str.maketrans('', '', PUNCT)).lower() # remove punctuation and set to lowercase
    post = re.sub(PATTERN,'', post) # remove non printable chars
    post = ' '.join([word for word in post.split() if word not in STOP_WORDS]) # remove stopwords
    post = ' '.join([SNOW.stem(word) for word in post.split()]) # stemming
    return post

open(OUT, 'w').close() # erase file berfore writing
with open(IN) as inFile, open(OUT, 'a') as outFile:
    inFileReader = csv.reader(inFile, delimiter=';')
    next(inFileReader) # skip header row
    outFileWriter = csv.writer(outFile, delimiter=';')
    outFileWriter.writerow(['id', 'post', 'title', 'tags'])

    for rowIn in inFileReader:
        newRow = [] # new list to append as a row to the output file
        newRow.append(rowIn[0]) # id remains the same
        newRow.append(cleanPost(rowIn[1])) # post gets cleaned
        newRow.append(cleanPost(rowIn[2])) # title gets cleaned
        newRow.append(rowIn[3]) # tags remain the same
        outFileWriter.writerow(newRow) # append new cleaned row to the output file  


