'''
This script labels each instance of the csv file from the previous phase by searching keywords in each post 
'''

import csv, string

# theese lists must have the same size, each file in the 'IN' list is corresponding to each file in the 'OUT' list, indexes must be coherent
IN = ['first clean csv file', 'second clean csv file', '...']
OUT = ['first labelled csv file', 'second labelled csv file', '...']

# raw Parrot's emotions three levels dictionary
# removed liking, longing, contentment to avoid false positives
'''emotionsRaw = {
        'love': {'affection':['adoration', 'affection', 'love', 'fondness', 'attraction', 'caring', 'tenderness', 'compassion', 'sentimentality'],
                'lust':['arousal', 'desire', 'lust', 'passion', 'infatuation']},

        'joy': {'cheerfulness':['amusement', 'bliss', 'cheerfulness', 'gaiety', 'glee', 'jolliness', 'joviality', 'joy', 'delight', 'enjoyment', 'gladness', 'happiness', 'jubilation', 'elation', 'satisfaction', 'ecstasy', 'euphoria'],
                'zest':['enthusiasm', 'zeal', 'zest', 'excitement', 'thrill', 'exhilaration'],
                'contentment':['pleasure'], 
                'pride':['pride', 'triumph'],
                'optimism':['eagerness', 'hope', 'optimism'],
                'enthrallment':['enthrallment', 'rapture'], 
                'relief':['relief']},

        'surprise': {'surprise':['amazement', 'surprise', 'astonishment']},

        'anger': {'irritation':['aggravation', 'irritation', 'agitation', 'annoyance', 'grouchiness', 'grumpiness'],
                'exasperation':['exasperation', 'frustration'],
                'rage':['anger', 'rage', 'outrage', 'fury', 'wrath', 'hostility', 'ferocity', 'bitterness', 'hate', 'loathing', 'scorn', 'spite', 'vengefulness', 'dislike', 'resentment'],
                'disgust':['disgust', 'revulsion', 'contempt'],
                'envy':['envy', 'jealousy'],
                'torment':['torment']},

        'sadness': {'suffering':['agony', 'suffering', 'hurt', 'anguish'],
                    'sadness':['depression', 'despair', 'hopelessness', 'gloom', 'glumness', 'sadness', 'unhappiness', 'grief', 'sorrow', 'woe', 'misery', 'melancholy'],
                    'disappointment':['dismay', 'disappointment', 'displeasure'],
                    'shame':['guilt', 'shame', 'regret', 'remorse'],
                    'neglect':['alienation', 'isolation', 'neglect', 'loneliness', 'rejection', 'homesickness', 'defeat', 'dejection', 'insecurity', 'embarassment', 'humiliation', 'insult'],
                    'sympathy':['pity', 'sympathy']},

        'fear': {'nervousness':['anxiety', 'nervousness', 'tenseness', 'uneasiness', 'apprehension', 'worry', 'distress', 'dread'],
                'horror':['alarm', 'shock', 'fear', 'fright', 'horror', 'terror', 'panic', 'hysteria', 'mortification']}
}'''

# stemmed Parrot's emotions two levels dictionary, the previous third and second levels are merged into one
# removed like, long, content to avoid false positives
emotionsStemmedFlat = {
        'love': ['ador', 'affect', 'fond', 'attract', 'care', 'tender', 'compass', 'sentiment', 'arous', 'desir', 'lust', 'passion', 'infatu'],
        'joy': ['amus', 'bliss', 'cheer', 'gaieti', 'glee', 'jolli', 'jovial', 'delight', 'enjoy', 'glad', 'happi', 'jubil', 'elat', 'satisfact', 'ecstasi', 'euphoria', 'enthusiasm', 'zeal', 'zest', 'excit', 'thrill', 'exhilar', 'pleasur', 'pride', 'triumph', 'eager', 'hope', 'optim', 'enthral', 'raptur', 'relief'],
        'surpris':['amaz', 'astonish'],
        'anger': ['aggrav', 'irrit', 'agit', 'annoy', 'grouchi', 'grumpi', 'exasperation', 'exasper', 'frustrat', 'rage', 'outrag', 'furi', 'wrath', 'hostil', 'feroc', 'bitter', 'hate', 'loath', 'scorn', 'spite', 'veng', 'dislik', 'resent', 'disgust', 'revuls', 'contempt', 'envi', 'jealousi', 'torment'],
        'sad': ['agoni', 'suffer', 'hurt', 'anguish', 'depress', 'despair', 'hopeless', 'gloom', 'glum', 'sad', 'unhappi', 'grief', 'sorrow', 'woe', 'miseri', 'melancholi', 'dismay', 'disappoint', 'displeasur', 'guilt', 'shame', 'regret', 'remors', 'alien', 'isol', 'neglect', 'loneli', 'reject', 'homesick', 'defeat', 'deject', 'insecur', 'embarass', 'humili', 'insult', 'piti', 'sympathi'],
        'fear': ['anxieti', 'nervous', 'tens', 'uneasi', 'apprehens', 'worri', 'distress', 'dread', 'alarm', 'shock', 'fright', 'horror', 'terror', 'panic', 'hysteria', 'mortif']
}


EMOTIONS = emotionsStemmedFlat.items()
# create lists with first and second level words
primaryPos = ['love', 'joy', 'surpris']
primaryNeg = ['anger', 'sad', 'fear']
secondaryPos = []
for secondary in [emotionsStemmedFlat['love'], emotionsStemmedFlat['joy'], emotionsStemmedFlat['surpris']]:
    for element in secondary:
        secondaryPos.append(element)

secondaryNeg = []
for secondary in [emotionsStemmedFlat['anger'], emotionsStemmedFlat['sad'], emotionsStemmedFlat['fear']]:
    for element in secondary:
        secondaryNeg.append(element)


def searchWord(post):
    firstLabel = ''
    secondLabel = ''
    thirdLabel = ''
    keywordToRemove = ''
    primaryList = [] 
    keywordList = []
    posCount = 0
    negCount = 0
    loveCount = 0
    joyCount = 0
    surpriseCount = 0
    angerCount = 0
    sadCount = 0
    fearCount = 0
    for word in post.split():
        for primary, secondaryList in EMOTIONS:
            # look for primary emotions first
            if word == primary:
                if primary == 'love':
                    posCount += 1
                    loveCount += 1
                elif primary == 'joy':
                    posCount += 1
                    joyCount += 1
                elif primary == 'surpris':
                    posCount += 1
                    surpriseCount += 1
                elif primary == 'anger':
                    negCount += 1
                    angerCount += 1
                elif primary == 'sad':
                    negCount += 1
                    sadCount += 1
                elif primary == 'fear':
                    negCount += 1
                    fearCount += 1

                if primary not in primaryList:
                    primaryList.append(primary)
                if primary not in keywordList:
                    keywordList.append(primary)
            # if no primary emotions were found look for secondary
            else:
                for keyword in secondaryList:
                    if word == keyword:
                        if primary == 'love':
                            posCount += 1
                            loveCount += 1
                        elif primary == 'joy':
                            posCount += 1
                            joyCount += 1
                        elif primary == 'surpris':
                            posCount += 1
                            surpriseCount += 1
                        elif primary == 'anger':
                            negCount += 1
                            angerCount += 1
                        elif primary == 'sad':
                            negCount += 1
                            sadCount += 1
                        elif primary == 'fear':
                            negCount += 1
                            fearCount += 1

                        if primary not in primaryList:
                            primaryList.append(primary)
                        if keyword not in keywordList:
                            keywordList.append(keyword)
                          
    if posCount > negCount:
        firstLabel = 'subjective'
        secondLabel = 'positive'
    elif posCount < negCount:
        firstLabel = 'subjective'
        secondLabel = 'negative'
    else :  # if positive keywords are the same numbers as the negative or if no keywords are found label it as neutral
        firstLabel = 'objective'
        secondLabel = ''


    found = False
    # find what keyword to remove
    if secondLabel == 'positive':
        for word in keywordList:
            if word in primaryPos:
                keywordToRemove = word
                found = True
                break
        if found == False:
            for word in keywordList:
                if word in secondaryPos:
                    keywordToRemove = word
                    break

    if secondLabel == 'negative':
        for word in keywordList:
            if word in primaryNeg:
                keywordToRemove = word
                found = True
                break
        if found == False:
            for word in keywordList:
                if word in secondaryNeg:
                    keywordToRemove = word
                    break

    
                                                                
    # if not found return an array with empty values
    retVal = [firstLabel, secondLabel, primaryList, keywordToRemove]
    return retVal



def arrayToAppend(row, list):
    newRow = []
    newRow.append(list[0]) # first label
    newRow.append(list[1]) # second label    
    primary = ', '.join(list[2])
    newRow.append(primary) # third label
    newRow.append(row[1]) # post
    return newRow





for fIn, fOut in zip(IN, OUT):
    open(fOut, 'w').close() # overwrite files with emptyness

    with open(fIn, 'r') as inFile, open(fOut, 'a') as outFile:
        inReader = csv.reader(inFile, delimiter=';')
        next(inReader) # skip header
        outWriter = csv.writer(outFile, delimiter=';')
        outWriter.writerow(['first label', 'second label', 'third label', 'post']) # write header

        for row in inReader:
            # search keyword
            listFromSearch = searchWord(row[1])
            # remove keyword from post if it's not objective
            if listFromSearch[0] != 'objective':
                row[1] = row[1].replace(listFromSearch[3], '')
                # doing so creates a duplicate whitespace, remove it
                row[1] = " ".join(row[1].split())
            outWriter.writerow(arrayToAppend(row, listFromSearch))
    
    print('\nDone', fOut)

