'''
    This script takes the csv file cleaned previously as input and
    outputs a clean csv file
'''

import csv, string, re
from nltk.corpus import stopwords
from nltk.stem.snowball import SnowballStemmer

# csv files path
IN = 'almost clean csv file'
OUT = 'clean csv file'

# some costants to speed up the process
PATTERN = re.compile('[^\x00-\x7f]')
STOP_WORDS = stopwords.words('english')
SNOW = SnowballStemmer('english')
PUNCT = string.punctuation

def cleanPost(post): 
    post = post.translate(str.maketrans('', '', PUNCT)).lower() # remove punctuation and set to lowercase
    post = re.sub(PATTERN,'', post) # remove non printable chars
    post = ' '.join([word for word in post.split() if word not in STOP_WORDS]) # remove stopwords
    post = ' '.join([SNOW.stem(word) for word in post.split()]) # stemming
    return post


open(OUT, 'w').close()
with open(IN, 'r') as inFile, open(OUT, 'a') as outFile:
    reader = csv.reader(inFile, delimiter=';')
    next(reader) # skip header row
    writer = csv.writer(outFile, delimiter=';')
    writer.writerow(['id', 'tags', 'date', 'title', 'post'])

    for rowIn in reader:
        newRow = [] # new list to append to output file
        newRow.append(rowIn[0]) # id remains the same
        newRow.append(rowIn[3]) # tags remain the same
        newRow.append(rowIn[4]) # date remains the same
        newRow.append(cleanPost(rowIn[5])) # title gets cleaned
        newRow.append(cleanPost(rowIn[6])) # post gets cleaned
        writer.writerow(newRow) # append new cleaned row to output file  




